
clc
clearvars;
c=3e8;
format long;

%Radar Parameter
fc=10e9;        %Carrier Frequency 10GHz
B=25e6;         %Bandwidth 25MHz
fs=7.5e7;       %Sampling Frequency 75MSps
PRF=10e3;       %Pulse Repetition Frequency 10kHz
Tp=2e-5;        %Pulse Winth 20us


T=1/PRF;        %Pulse Repetition Time 100us
rou=c/2/fs;     %Space Sampling Interval 2m
D=B*Tp;         %Compession Ratio 500
gamma=B/Tp;     %Frequency Modulation Ratio 1.25e12
k=fs/B;         %Sampling Ratio 3
lamda=c/fc;     %Wavelength 3cm
vb=lamda/(2*T); %Blind Speed 150m/s

%Target Parameter
r0=10000;       %Initial Range 10km
nr0=r0/rou;     %Discretized Initial Range
v0=2000;        %Initial Velocity 2000m/s
a0=-100;        %Acceleration -100m/s

A1=1;           %Echo Amplitude. Normalized.    

%Target Parameter:Searching Parameter
rc=10000;       %Concerned Center of Range
ra=480;         %Range length concerned in the search
rg=1000;        %Data length needed in the search. rg>=ra+max(v0, vt)*TMax
N=round(rg/rou)+1;                  %Number of data length needed in the search 
ng_start=round((rc-rg/2)/rou);      %Start point of data needed
na_start=round((rc-ra/2)/rou);      %Start point of range searching region.
delta_r=c/(2*fs);                   %Range Searching Intervel.
Nr=length((rc-ra/2):delta_r:(rc+ra/2)); %Number of Range Cells 

v_min=1550;      v_max=2450;        %Velocity Scope
a_min=-190;a_max=190;               %Acceleration Scope
Tmax=0.1;                           %Time Period
TM_line=(T:T:Tmax)';                %Slow time sequence
TM=Tmax;                            %Time Period
M=round(TM/T);                      %Pulse Number
delta_v=lamda/(2*TM);               %Velocity Searching Interval
Nv=length(v_min:delta_v:v_max);     %Number of velocity Cells 

delta_a=lamda/Tmax^2;               %Acceleration Searching Interval
Na=length(a_min:delta_a:a_max);     %Number of acceleration Cells 

%脉压输出结果
m=(1:M)';n=1:N;                     %Index of slow time and fast time.
SimTrack=r0+v0.*TM_line+1/2*a0.*TM_line.^2;     %Generate the motion of the target.
sc=A1*sinc(1/k*(zeros(M,N)+n-1+ng_start-SimTrack(1:M)./rou)).*exp(-1j*4*pi*SimTrack(1:M)./lamda);    %Generate the echo of the target.

qmax=min(floor((v_max-v0)/vb),floor((r0-(rc-ra/2))/(M*lamda/4)));   %minimal order of BSSL
qmin=max(ceil(v_min-v0)/vb,ceil((r0-(rc+ra/2))/(M*lamda/4)));       %maximal order of BSSL
q=(qmin:qmax)';                                                     %order of all the BSSLs
realPeak=[r0,v0,a0]+[-q*M*lamda/4,q*lamda/(2*T),q*0];               %Theoritical location of main lobe and center of BSSLs
%This loop generates the theoritical location and spectrum values of all
%the main lobe and center of BSSLs, and save it.
for qq=1:length(realPeak)                                           
    r0_p=realPeak(qq,1);v0_p=realPeak(qq,2);a0_p=realPeak(qq,3);    
    Ac_p=v0_p*m*T+0.5*a0_p*T*T*m.*m;
    IND_n=round((r0_p+Ac_p-(rc-rg/2))/rou)+1;
    %IND=sub2ind(size(sc),m,round(1+(index_n_p-1)*delta_r/rou)-ng_start+na_start+round(Ln_p/rou));
    IND=sub2ind(size(sc),m,IND_n);
    sc_p_sum_a=-abs(sum(sc(IND).*exp(1j*4*pi*Ac_p/lamda)));
    realPeak(qq,4)=sc_p_sum_a;
end
save("realPeak_GRFT.mat","realPeak");
% PSO-GRFT
Dim=3;                                                              %Motion Parameter Dimension is 3, when motion order is 2. 
x_lim=[rc-ra/2,rc+ra/2,v_min,v_max,a_min,a_max];                    %Motion Parameter Scope
xmin=x_lim([1 3 5]);xmax=x_lim([2 4 6]);w_range=[0.3,1.1];x0=xmin;  %w_range is the range of weight used in the PSO-based algorithm.
delta=[delta_r,delta_v,delta_a];
NN=100;xm=zeros(3,NN);                                              %Number of Monte Carlo experiments 
rng(42);
tic

for ii=1:NN

    objfun = @(x)fun_GRFT(x,sc);                                    %This function calculates the amplitude of GRFT spectrum at location x.
    

    options = optimoptions('particleswarm','SwarmSize',200,...      %Parameters of PSO-based algorithm.Particle swarm size 200, self adjust weight 2, social adjust weight 0.5, range of weight [0.3 1.1]
        'SelfAdjustmentWeight',2,'SocialAdjustmentWeight',0.5,'InertiaRange',...
        w_range,'PlotFcn','pswplotbestf',"Display","off");
    

    [solution,objectiveValue,exitFlag,output] = particleswarm(objfun,Dim,xmin,xmax,options); %call the PSO-based function. In the algorithm 
    

    %clearvars objfun options
    xm(:,ii)=solution ;
    fv(ii)=objectiveValue;
    exitF(ii)=exitFlag;
    Iterations(ii)=output.iterations;
end
data.xm=xm;
data.fv=fv;
data.exitF=exitF;
data.Iterations=Iterations;
data.endtime=toc;

[ym,indexym]=sortrows(data.xm',2);
r0_p=ym(:,1); v0_p=ym(:,2); a0_p=ym(:,3);sc_p_sum_a=fv(indexym)';exitFg=exitF(indexym)';
data.accurate_percent=sum(abs(v0_p-v0)<5*lamda/(2*TM))/NN
data.mainlobe_percent=sum(abs(v0_p-v0)<2*lamda/(2*T))/NN
data.sc_p_sum_alpha1=max(abs(sc_p_sum_a))
h=figure;h.Position=[600,400,350,300];
scatter(r0_p,v0_p);hold on;
xlabel('{\itr}_0(m)');ylabel('{\itv}_0(m/s)');
%save('结果文件\data_particle_algorithm_4','data');

